OtoNumModifier {
	// Main var
	classvar <currentValue = 0, <>docStringNumberRange, <>docStringNumberStart, <>target;
	
	// GUI var
	classvar <>window, <>userview, <>closeButton, <>numberbox;
	classvar <>plusGlab, <>min1Button, <>plus1Button;
	classvar <>mulGlab, <>div2Button, <>mul2Button;
	classvar <>intButton, <>recipButton;
	classvar <>codeButton, <>codeInput;
	
	// MouseGrab var
	classvar <>mouseDownPoint, <>mouseDownValue;
	
	*new {
		var winbounds;
		
		if(window.notNil, {
			winbounds = window.bounds;
			window.close;
			{this.createGUI(winbounds)}.defer(0.2);
		},{
			this.createGUI;
		});
	}
	
	*resetAll {
		currentValue = nil; docStringNumberRange = nil; docStringNumberStart = nil;
		window = nil; userview = nil; closeButton = nil; numberbox = nil;
		plusGlab = nil; min1Button = nil; plus1Button = nil;
		mulGlab = nil; div2Button = nil; mul2Button = nil;
		intButton = nil; recipButton = nil;
		codeButton = nil; codeInput = nil;
		
		Document.initAction = nil;
		Document.allDocuments.do{|d| d.mouseDownAction = nil};
	}
	
	*createGUI {| winbounds |

		// Reset All
		this.resetAll;
		
		// Create GUI
		
		window = GUI.window.new("OtoNumModifier", winbounds ? Rect(200, 200, 240, 80), false, true)
			.alpha_(1.0)
			.onClose_({{OtoNumModifier.resetAll}.defer(0.1)})
			.alwaysOnTop_(true)
			.front;
	
		userview = GUI.userView.new(window, window.view.bounds)
			.mouseBeginTrackFunc_({ | me, x, y, modifiers |
				mouseDownPoint = Point(x, y);
			})
			.mouseTrackFunc_({ | me, x, y, modifiers |
				window.bounds = window.bounds.moveBy(x - mouseDownPoint.x, mouseDownPoint.y - y);
			})
			.mouseEndTrackFunc_({ | me, x, y, modifiers |
				window.bounds = window.bounds.moveBy(x - mouseDownPoint.x, mouseDownPoint.y - y);
				mouseDownPoint = Point(x, y);
			})
			.canFocus_(false);
	
		window.view
			// .keydownAction_({| view, char, modifiers, unicode, keycode | })
			.decorator_(FlowLayout(window.view.bounds));
		
			
		closeButton = GUI.button.new(window, Rect(0, 0, 36, 20))
			.states_([
				["X", Color.white, Color.black]
			])
			.font_(Font("Monaco", 10))
			.action_({window.close});
		
		
		numberbox = GUI.numberBox.new(window, Rect(0, 0, 192, 20) )
			.font_(Font("Monaco", 18))
			.stringColor_(Color.black)
			.value_(currentValue)
			.focus
			.action_({this.currentValue = numberbox.value});
		
		window.view.decorator.nextLine;
		
		window.view.decorator.gap = Point(0, 4);
		
		plusGlab = GUI.staticText.new(window, Rect(0, 0, 36, 20))
			.enabled_(true).canFocus_(true)
			.background_(Color.blue(1.0, 0.5)).stringColor_(Color.white)
			.font_(Font("Monaco", 10)).align_('center').string_("+")
			.mouseDownAction_({ | me, x, y, modifiers |
				mouseDownValue = currentValue;
				mouseDownPoint = Point(x, y);
			})
			.mouseMoveAction_({ | me, x, y, modifiers |					this.currentValue = ( (mouseDownPoint.y - y).cubed / 131072 + mouseDownValue );
				
			})
			.mouseUpAction_({ | me, x, y, modifiers |
				this.currentValue = ( (mouseDownPoint.y - y).cubed / 131072 + mouseDownValue );
				
			});
		
		min1Button = GUI.button.new(window, Rect(0, 0, 24, 20))
			.states_([
				["-1", Color.white, Color.blue(1.0, 0.5)]
			])
			.font_(Font("Monaco", 10))
			.action_({this.currentValue = currentValue-1;});
		window.view.decorator.gap = Point(4, 4);
		
		plus1Button = GUI.button.new(window, Rect(0, 0, 24, 20))
			.states_([
				["+1", Color.white, Color.blue(1.0, 0.5)]
			])
			.font_(Font("Monaco", 10))
			.action_({this.currentValue = currentValue+1;});
		
		
		window.view.decorator.gap = Point(0, 4);
		mulGlab = GUI.staticText.new(window, Rect(0, 0, 36, 20))
			.enabled_(true).canFocus_(true)
			.background_(Color.red(1.0, 0.5)).stringColor_(Color.white)
			.font_(Font("Monaco", 10)).align_('center').string_("*")
			.mouseDownAction_({ | me, x, y, modifiers |
				mouseDownValue = currentValue;
				mouseDownPoint = Point(x, y);
			})
			.mouseMoveAction_({ | me, x, y, modifiers |
				this.currentValue = ((2 ** ((mouseDownPoint.y - y)/128)) * mouseDownValue);
				
			})
			.mouseUpAction_({ | me, x, y, modifiers |
				this.currentValue = ((2 ** ((mouseDownPoint.y - y)/128)) * mouseDownValue);
				
			});
			
		
		div2Button = GUI.button.new(window, Rect(0, 0, 24, 20))
			.states_([
				["/2", Color.white, Color.red(1.0, 0.5)]
			])
			.font_(Font("Monaco", 10))
			.action_({this.currentValue = currentValue/2;});
		window.view.decorator.gap = Point(4, 4);
		
		mul2Button = GUI.button.new(window, Rect(0, 0, 24, 20))
			.states_([
				["*2", Color.white, Color.red(1.0, 0.5)]
			])
			.font_(Font("Monaco", 10))
			.action_({this.currentValue = currentValue*2;});		
		window.view.decorator.gap = Point(4, 0);
		intButton = GUI.button.new(window, Rect(0, 0, 24, 20))
			.states_([
				["Int", Color.white, Color.green(0.5, 0.5)]
			])
			.font_(Font("Monaco", 10))
			.action_({this.currentValue = currentValue.round(1);});
		
		window.view.decorator.gap = Point(4, 4);
		recipButton = GUI.button.new(window, Rect(0, 0, 24, 20))
			.states_([
				["1/", Color.white, Color.green(0.5, 0.5)]
			])
			.font_(Font("Monaco", 10))
			.action_({this.currentValue = currentValue.reciprocal;});
		
		
		window.view.decorator.nextLine;
		
		codeButton = GUI.button.new(window, Rect(0, 0, 36, 15))
			.states_([
				["x =", Color.white, Color.gray(0.4, 0.5)]
			])
			.font_(Font("Monaco", 10))
			.action_({
				this.interpretCodeIn;
			});
		
		
		codeInput = GUI.textView.new(window, Rect(0, 0, 192, 15))
			.font_(Font("Monaco", 10)).string_("x")
			.usesTabToFocusNextView_(true)
			.keyDownAction_({| view, char, mod, unicode, keycode |
				if(unicode == 13, {
					{codeInput.string = codeInput.string.reject({| item | item == $\n })}.defer(0.1)
				});
				if(keycode == 76, { this.interpretCodeIn; });
				if(keycode == 36, { this.interpretCodeIn; });
			})
			.enterInterpretsSelection_(false);
		
		
		
		Document.initAction = {|d| d.mouseDownAction = {|doc|
			OtoNumModifier.detectSelectedNumber(doc);
		}};	
		Document.allDocuments.do{|d| d.mouseDownAction = {|doc|
			OtoNumModifier.detectSelectedNumber(doc);
		}};
	}
	
	*currentValue_ {| value |
		if(window.notNil && target.notNil && value != currentValue, {
			currentValue = value;
			numberbox.value = currentValue;
			
			if(target.class == Document, {
				target.string_(currentValue.asString, docStringNumberStart, docStringNumberRange);
				docStringNumberRange = currentValue.asString.size;
			});
		});
	}
	
	*interpretCodeIn {
		("OtoNumModifier.currentValue = " ++ codeInput.string.replace("x", "OtoNumModifier.currentValue") ++ ";").interpret;
	}
	
	*detectSelectedNumber {|doc|
		var selst, selsz, break, lefti, righti, charl, charr, dot, minus;
		
		selst = doc.selectionStart;
		selsz = doc.selectionSize;
		
		
		// ScanRight
		break = false;
		righti = if(selst < doc.string.size, {selst}, {break = true; selst});
		dot = nil;
		minus = nil;
		
		while ({break.not}, {
			charr = doc.string.at(righti);
			
			if(charr.isDecDigit || 
				if((charr == $.), {
					if(dot.isNil && doc.string.at(righti+1).isDecDigit && doc.string.at(righti-1).isDecDigit, {dot = righti; true;}, {false})
				}, {false}) ||
				
				if((charr == $-), {
					if(minus.isNil && doc.string.at(righti+1).isDecDigit &&
						([$\n, $ , $t, $(, ${, $[, $,, $., $;, $;, $", $', nil].indexOfEqual(doc.string.at(righti-1)).notNil)
						, {minus = righti; true}, {false})
				}, {false}),
			{
				righti = righti + 1;
				if(righti >= doc.string.size, {break = true;});
			},{
				break = true;
			});
			
		});
		
		// ScanLeft
		break = false;
		lefti = if(selst>1, {selst-1}, {break = true; -1});
		
		while ({break.not}, {
			charl = doc.string.at(lefti);
			
			if(charl.isDecDigit ||
				if((charl == $.), {
					if(dot.isNil && doc.string.at(lefti+1).isDecDigit && doc.string.at(lefti-1).isDecDigit, {dot = lefti; true;}, {false})
				}, {false}) ||
				
				if((charl == $-), {
					if(minus.isNil && doc.string.at(lefti+1).isDecDigit &&
						([$\n, $ , $t, $(, ${, $[, $,, $., $;, $;, $", $', nil].indexOfEqual(doc.string.at(lefti-1)).notNil)
						, {minus = lefti; true}, {false})
				}, {false}),
			{
				lefti = lefti - 1;
				if(lefti < 0, {break = true; });
			},{
				break = true;
			});
			
		});
		
		if( 1 < (righti - lefti), {
			target = doc;
			currentValue = doc.string(lefti+1, righti - lefti - 1).asFloat;
			numberbox.value = currentValue;
			
			docStringNumberStart = lefti+1;
			docStringNumberRange = righti - lefti - 1;
		},{
			target = nil;
			currentValue = 0;
			numberbox.value = "Not a Number";
			docStringNumberStart = nil;
			docStringNumberRange = nil;
		});
	}
}